package com.laohuai.appdemo.customui.ui;

import java.util.Date;

import android.content.Context;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.LinearInterpolator;
import android.view.animation.RotateAnimation;
import android.widget.AbsListView;
import android.widget.AbsListView.OnScrollListener;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.laohuai.appdemo.customui.R;

/**
 * 可以下拉显示的ListView
 * 
 * @author Jiang Hao
 * QQ:36539608
 * Mail: jianghao3616@gmail.com
 * 欢迎加入Android群讨论:124542555
 * 
 */
public class HeadListView extends ListView implements OnScrollListener {

    private static final String TAG = "HeadListView";

    private static final int STATE_IDLE = 0;
    private static final int STATE_PULL = 1;
    private static final int STATE_HOLD = 2;
    private static final int STATE_ROLLBACK = 3;
    private static final int STATE_RELEASE_To_HOLD = 4;
    private static final int STATE_HOLD_RELEASE_To_HOLD = 5;
    private static final int STATE_HOLD_RELEASE_To_ROLLBACK = 6;
    private static final int STATE_RELEASE_To_ROLLBACK = 7;
    private static final int STATE_ROLLBACK_RELEASE_To_HOLD = 8;
    private static final int STATE_ROLLBACK_RELEASE_To_ROLLBACK = 9;
    private static final int STATE_DONE = 10;

    private static DisplayMetrics mDisplayMetrics;

    // 实际的padding的距离与界面上偏移距离的比例
    private final static int RATIO = 3;

    private LayoutInflater mInflater;

    private LinearLayout mHeadView;

    private TextView tipsTextview;
    private ImageView arrowImageView;
    private TextView lastUpdatedTextView;
    private ProgressBar progressBar;

    private RotateAnimation animation;
    private RotateAnimation reverseAnimation;

    // 用于保证startY的值在一个完整的touch事件中只被记录一次
    private boolean isRecored;

    private int mHeadContentWidth;
    private int mHeadContentHeight;

    private int mOffsetThreshold;

    private int mStartY;
    private int mFirstItemIndex;

    private int mState;

    public HeadListView(Context context) {
        this(context, null);
    }

    public HeadListView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public HeadListView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init(context);
        // TODO Auto-generated constructor stub

    }

    private void init(Context context) {
        setCacheColorHint(context.getResources().getColor(R.color.transparent));
        mInflater = LayoutInflater.from(context);
        mDisplayMetrics = context.getResources().getDisplayMetrics();
        setOnScrollListener(this);

        animation = new RotateAnimation(0, -180,
                RotateAnimation.RELATIVE_TO_SELF, 0.5f,
                RotateAnimation.RELATIVE_TO_SELF, 0.5f);
        animation.setInterpolator(new LinearInterpolator());
        animation.setDuration(250);
        animation.setFillAfter(true);

        reverseAnimation = new RotateAnimation(-180, 0,
                RotateAnimation.RELATIVE_TO_SELF, 0.5f,
                RotateAnimation.RELATIVE_TO_SELF, 0.5f);
        reverseAnimation.setInterpolator(new LinearInterpolator());
        reverseAnimation.setDuration(200);
        reverseAnimation.setFillAfter(true);

        mState = STATE_IDLE;
    }

    // 此方法直接照搬自网络上的一个下拉刷新的demo，此处是“估计”headView的width以及height
    private void measureView(View child) {
        ViewGroup.LayoutParams p = child.getLayoutParams();
        if (p == null) {
            p = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT);
        }
        int childWidthSpec = ViewGroup.getChildMeasureSpec(0, 0 + 0, p.width);
        int lpHeight = p.height;
        int childHeightSpec;
        if (lpHeight > 0) {
            childHeightSpec = MeasureSpec.makeMeasureSpec(lpHeight,
                    MeasureSpec.EXACTLY);
        } else {
            childHeightSpec = MeasureSpec.makeMeasureSpec(0,
                    MeasureSpec.UNSPECIFIED);
        }
        child.measure(childWidthSpec, childHeightSpec);
    }

    @Override
    public void onScroll(AbsListView arg0, int firstVisiableItem, int arg2,
            int arg3) {
        mFirstItemIndex = firstVisiableItem;
    }

    @Override
    public void onScrollStateChanged(AbsListView arg0, int arg1) {
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        // TODO Auto-generated method stub
        switch (event.getAction()) {
        case MotionEvent.ACTION_DOWN:
            Log.v(TAG,"ACTION_DOWN mFirstItemIndex "+mFirstItemIndex+" isRecored "+isRecored);
            if (mFirstItemIndex == 0 && !isRecored) {
                isRecored = true;
                mStartY = (int) event.getY();
                Log.v(TAG, "记录按下坐标 "+mStartY);
            }
            break;
        case MotionEvent.ACTION_MOVE:
            int tempY = (int) event.getY();
            if (!isRecored && mFirstItemIndex == 0) {
                isRecored = true;
                mStartY = tempY;
            }
            // 可以松手去刷新了
            if (mState == STATE_RELEASE_To_HOLD) {

                setSelection(0);
                // 往上推了，推到了屏幕足够掩盖head的程度，但是还没有推到全部掩盖的地步
                if (((tempY - mStartY) / RATIO < mOffsetThreshold)
                        && (tempY - mStartY) > 0) {
                    mState = STATE_RELEASE_To_ROLLBACK;
                    changeHeaderViewByState();
                } else if (tempY - mStartY <= -20) {// 一下子推到顶了
                    mState = STATE_RELEASE_To_ROLLBACK;
                    changeHeaderViewByState();
                } else {// 往下拉了，或者还没有上推到屏幕顶部掩盖head的地步
                        // 不用进行特别的操作，只用更新paddingTop的值就行了，这里改变下文字，测试用
                    changeHeaderViewByState();
                }
            } else if (mState == STATE_PULL) {
                setSelection(0);
                // 下拉到一定高度可以进入RELEASE_TO_REFRESH的状态
                if ((tempY - mStartY) / RATIO >= mOffsetThreshold) {
                    mState = STATE_RELEASE_To_HOLD;
                    changeHeaderViewByState();

                } else if (tempY - mStartY <= -20) {// 上推到顶了
                    mState = STATE_RELEASE_To_ROLLBACK;
                    changeHeaderViewByState();
                }
            } else if (mState == STATE_HOLD
                    || mState == STATE_HOLD_RELEASE_To_HOLD) {
                Log.v(TAG, "起始状态 固定或者松手固定 ");
                setSelection(0);
                // 下拉到一定高度可以进入RELEASE_TO_REFRESH的状态
                if (tempY - mStartY > 0) {
                    mState = STATE_HOLD_RELEASE_To_HOLD;
                } else if (tempY - mStartY < -20) {// 上推到顶了
                    mState = STATE_HOLD_RELEASE_To_ROLLBACK;
                }
            } else if (mState == STATE_RELEASE_To_ROLLBACK) {
                // 下拉到一定高度可以进入RELEASE_TO_REFRESH的状态
                Log.v(TAG, "释放后收回 ");
                if ((tempY - mStartY) / RATIO >= mOffsetThreshold) {
                    mState = STATE_RELEASE_To_HOLD;
                    changeHeaderViewByState();

                } else if (tempY - mStartY <= -20) {// 上推到顶了
                    mState = STATE_RELEASE_To_ROLLBACK;
                    changeHeaderViewByState();
                }
            } else if (mState == STATE_HOLD_RELEASE_To_ROLLBACK
                    || mState == STATE_ROLLBACK_RELEASE_To_HOLD) {
                if ((tempY - mStartY) / RATIO >= 0) {
                    mState = STATE_ROLLBACK_RELEASE_To_HOLD;
                } else if (tempY - mStartY <= -20) {// 上推到顶了
                    mState = STATE_HOLD_RELEASE_To_ROLLBACK;
                }

            } else if (mState == STATE_IDLE || mState == STATE_DONE) {
                if (tempY - mStartY > 0) {
                    mState = STATE_PULL;
                    changeHeaderViewByState();
                }
            }

            // 更新headView的size
            if (mState == STATE_PULL) {
                mHeadView.setPadding(0, -1 * mHeadContentHeight
                        + (tempY - mStartY) / RATIO, 0, 0);
            }

            // 更新headView的paddingTop
            if (mState == STATE_RELEASE_To_HOLD
                    || mState == STATE_RELEASE_To_ROLLBACK
                    || mState == STATE_ROLLBACK_RELEASE_To_ROLLBACK) {
                mHeadView.setPadding(0, (tempY - mStartY) / RATIO
                        - mHeadContentHeight, 0, 0);
            } else if (mState == STATE_HOLD_RELEASE_To_HOLD
                    || mState == STATE_HOLD_RELEASE_To_ROLLBACK
                    || mState == STATE_ROLLBACK_RELEASE_To_HOLD) {
                mHeadView.setPadding(0, (tempY - mStartY) / RATIO, 0, 0);

            }

            break;
        case MotionEvent.ACTION_UP:
            if (mState == STATE_DONE) {
                // 什么都不做
            }
            if (mState == STATE_PULL) {
                mState = STATE_ROLLBACK;
                changeHeaderViewByState();
            }
            if (mState == STATE_RELEASE_To_HOLD
                    || mState == STATE_HOLD_RELEASE_To_HOLD
                    || mState == STATE_ROLLBACK_RELEASE_To_HOLD) {
                mState = STATE_HOLD;
                changeHeaderViewByState();
            }
            if (mState == STATE_RELEASE_To_ROLLBACK
                    || mState == STATE_HOLD_RELEASE_To_ROLLBACK
                    || mState == STATE_ROLLBACK_RELEASE_To_ROLLBACK) {
                mState = STATE_ROLLBACK;
                changeHeaderViewByState();
            }

            isRecored = false;
            break;
        }

        return super.onTouchEvent(event);
    }

    // 当状态改变时候，调用该方法，以更新界面
    private void changeHeaderViewByState() {
        switch (mState) {
        case STATE_RELEASE_To_HOLD:
            tipsTextview.setText("松开固定显示");
        case STATE_HOLD:
            mHeadView.setPadding(0, 0 * mHeadContentHeight, 0, 0);
            progressBar.setVisibility(View.GONE);
            arrowImageView.setImageResource(R.drawable.arrow);
            lastUpdatedTextView.setVisibility(View.VISIBLE);

            arrowImageView.clearAnimation();
            arrowImageView.startAnimation(animation);

            tipsTextview.setText("显示全部Head");
            break;
        case STATE_PULL:
            progressBar.setVisibility(View.GONE);
            tipsTextview.setVisibility(View.VISIBLE);
            lastUpdatedTextView.setVisibility(View.VISIBLE);
            arrowImageView.clearAnimation();
            arrowImageView.setVisibility(View.VISIBLE);
            // 是由RELEASE_To_REFRESH状态转变来的

            tipsTextview.setText("下拉显示全部");
            break;

        case STATE_DONE:
            mHeadView.setPadding(0, 0 * mHeadContentHeight, 0, 0);
            // headView.setVisibility(View.INVISIBLE);
            progressBar.setVisibility(View.GONE);
            arrowImageView.clearAnimation();
            arrowImageView.setImageResource(R.drawable.arrow);
            tipsTextview.setText("下拉刷新");
            lastUpdatedTextView.setVisibility(View.VISIBLE);

            break;
        case STATE_ROLLBACK:
            mHeadView.setPadding(0, -1 * mHeadContentHeight, 0, 0);
            // headView.setVisibility(View.INVISIBLE);
            progressBar.setVisibility(View.GONE);
            arrowImageView.clearAnimation();
            arrowImageView.setImageResource(R.drawable.arrow);
            tipsTextview.setText("收回HeadView");
            lastUpdatedTextView.setVisibility(View.VISIBLE);

            mState = STATE_IDLE;
            break;

        }
    }

    public interface OnRefreshListener {
        public void onRefresh();
    }

    public void onRefreshComplete() {
        mState = STATE_DONE;
        lastUpdatedTextView.setText("最近更新:" + new Date().toLocaleString());
        changeHeaderViewByState();
    }

    public void setAdapter(BaseAdapter adapter) {
        lastUpdatedTextView.setText("最近更新:" + new Date().toLocaleString());
        super.setAdapter(adapter);
    }

    public void setHeadView(int layout) {
        setHeadView(layout, 0);
    }

    public void setHeadView(int layout, int offset) {
        mHeadView = (LinearLayout) mInflater.inflate(layout, null);
        addHeaderView(mHeadView, null, false);

        arrowImageView = (ImageView) mHeadView
                .findViewById(R.id.head_arrowImageView);
        arrowImageView.setMinimumWidth(70);
        arrowImageView.setMinimumHeight(50);
        progressBar = (ProgressBar) mHeadView
                .findViewById(R.id.head_progressBar);
        tipsTextview = (TextView) mHeadView
                .findViewById(R.id.head_tipsTextView);
        lastUpdatedTextView = (TextView) mHeadView
                .findViewById(R.id.head_lastUpdatedTextView);

        measureView(mHeadView);
        mHeadContentHeight = mHeadView.getMeasuredHeight();
        mHeadContentWidth = mHeadView.getMeasuredWidth();
        if (offset <= 0) {
            mOffsetThreshold = mHeadContentHeight;
        } else {
            mOffsetThreshold = offset;
        }

        mHeadView.setPadding(0, -1 * mHeadContentHeight, 0, 0);
        mHeadView.invalidate();

        Log.v(TAG, "width:" + mHeadContentWidth + " height:"
                + mHeadContentHeight);
    }

    public View getHeadView() {
        return mHeadView;
    }

    /**
     * Converts density independent pixel value to raw pixel value
     */
    private static int dipToPixels(float dipValue) {
        return (int) (mDisplayMetrics.density * dipValue + 0.5f);
    }
}
